@@ -21,7 +21,7 @@
 #include <utils/filesystemwatcher.h>
 #include <utils/hostosinfo.h>
 #include <utils/persistentsettings.h>
-#include <utils/process.h>
+#include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
 
 #include <nanotrace/nanotrace.h>
@@ -89,7 +89,8 @@ static PersistentSettingsWriter *m_writer = nullptr;
 class QtVersionManagerImpl : public QObject
 {
 public:
-    QtVersionManagerImpl()
+    QtVersionManagerImpl(QObject *parent)
+        : QObject(parent)
     {
         qRegisterMetaType<FilePath>();
 
@@ -97,7 +98,7 @@ public:
         m_fileWatcherTimer.setInterval(2000);
         connect(&m_fileWatcherTimer, &QTimer::timeout, this, [this] { updateFromInstaller(); });
 
-        connect(ToolChainManager::instance(), &ToolChainManager::toolChainsLoaded,
+        connect(ToolchainManager::instance(), &ToolchainManager::toolchainsLoaded,
                 this, &QtVersionManagerImpl::triggerQtVersionRestore);
     }
 
@@ -120,7 +121,8 @@ public:
 
     void updateDocumentation(const QtVersions &added,
                              const QtVersions &removed,
-                             const QtVersions &allNew);
+                             const QtVersions &allNew,
+                             bool updateBlockedDocumentation = false);
 
     void setNewQtVersions(const QtVersions &newVersions);
     QString qmakePath(const QString &qtchooser, const QString &version);
@@ -134,17 +136,25 @@ public:
     QTimer m_fileWatcherTimer;
 };
 
+static QObject *s_guard = nullptr;
+
+void Internal::setupQtVersionManager(QObject *guard)
+{
+    s_guard = guard;
+}
+
 QtVersionManagerImpl &qtVersionManagerImpl()
 {
-    static QtVersionManagerImpl theQtVersionManager;
-    return theQtVersionManager;
+    QTC_CHECK(s_guard);
+    static auto theQtVersionManager = new QtVersionManagerImpl(s_guard);
+    return *theQtVersionManager;
 }
 
 void QtVersionManagerImpl::triggerQtVersionRestore()
 {
     NANOTRACE_SCOPE("QtSupport", "QtVersionManagerImpl::triggerQtVersionRestore");
-    disconnect(ToolChainManager::instance(),
-               &ToolChainManager::toolChainsLoaded,
+    disconnect(ToolchainManager::instance(),
+               &ToolchainManager::toolchainsLoaded,
                this,
                &QtVersionManagerImpl::triggerQtVersionRestore);
 
@@ -163,8 +173,7 @@ void QtVersionManagerImpl::triggerQtVersionRestore()
         NANOTRACE_SCOPE("QtSupport", "QtVersionManagerImpl::qtVersionsLoaded");
         emit QtVersionManager::instance()->qtVersionsLoaded();
     }
-    emit QtVersionManager::instance()->qtVersionsChanged(
-        m_versions.keys(), QList<int>(), QList<int>());
+    emit QtVersionManager::instance()->qtVersionsChanged(m_versions.keys());
 
     const FilePath configFileName = globalSettingsFileName();
     if (configFileName.exists()) {
@@ -175,7 +184,7 @@ void QtVersionManagerImpl::triggerQtVersionRestore()
     } // exists
 
     const QtVersions vs = QtVersionManager::versions();
-    updateDocumentation(vs, {}, vs);
+    updateDocumentation(vs, {}, vs, /*updateBlockedDocumentation=*/true);
 }
 
 bool QtVersionManager::isLoaded()
@@ -221,7 +230,7 @@ bool QtVersionManagerImpl::restoreQtVersions()
         if (!key.view().startsWith(keyPrefix))
             continue;
         bool ok;
-        int count = key.toByteArray().mid(keyPrefix.count()).toInt(&ok);
+        int count = key.toByteArray().mid(keyPrefix.size()).toInt(&ok);
         if (!ok || count < 0)
             continue;
 
@@ -295,7 +304,7 @@ void QtVersionManagerImpl::updateFromInstaller(bool emitSignal)
         if (!key.view().startsWith(keyPrefix))
             continue;
         bool ok;
-        int count = key.toByteArray().mid(keyPrefix.count()).toInt(&ok);
+        int count = key.toByteArray().mid(keyPrefix.size()).toInt(&ok);
         if (!ok || count < 0)
             continue;
 
@@ -411,7 +420,7 @@ QList<QByteArray> QtVersionManagerImpl::runQtChooser(const QString &qtchooser, c
     p.start();
     p.waitForFinished();
     const bool success = p.exitCode() == 0;
-    return success ? p.readAllRawStandardOutput().split('\n') : QList<QByteArray>();
+    return success ? p.rawStdOut().split('\n') : QList<QByteArray>();
 }
 
 // Asks qtchooser for the qmake path of a given version
@@ -478,7 +487,7 @@ void QtVersionManager::addVersion(QtVersion *version)
     int uniqueId = version->uniqueId();
     m_versions.insert(uniqueId, version);
 
-    emit QtVersionManager::instance()->qtVersionsChanged(QList<int>() << uniqueId, QList<int>(), QList<int>());
+    emit QtVersionManager::instance()->qtVersionsChanged({uniqueId});
     qtVersionManagerImpl().saveQtVersions();
 }
 
@@ -486,7 +495,7 @@ void QtVersionManager::removeVersion(QtVersion *version)
 {
     QTC_ASSERT(version, return);
     m_versions.remove(version->uniqueId());
-    emit QtVersionManager::instance()->qtVersionsChanged(QList<int>(), QList<int>() << version->uniqueId(), QList<int>());
+    emit QtVersionManager::instance()->qtVersionsChanged({}, {version->uniqueId()});
     qtVersionManagerImpl().saveQtVersions();
     delete version;
 }
@@ -500,20 +509,34 @@ void QtVersionManager::registerExampleSet(const QString &displayName,
 
 using Path = QString;
 using FileName = QString;
-static QList<std::pair<Path, FileName>> documentationFiles(QtVersion *v)
+using DocumentationFile = std::pair<Path, FileName>;
+using DocumentationFiles = QList<DocumentationFile>;
+using AllDocumentationFiles = QHash<QtVersion *, DocumentationFiles>;
+
+static DocumentationFiles allDocumentationFiles(QtVersion *v)
 {
-    QList<std::pair<Path, FileName>> files;
+    DocumentationFiles files;
     const QStringList docPaths = QStringList(
         {v->docsPath().toString() + QChar('/'), v->docsPath().toString() + "/qch/"});
     for (const QString &docPath : docPaths) {
         const QDir versionHelpDir(docPath);
-        for (const QString &helpFile : versionHelpDir.entryList(QStringList("*.qch"), QDir::Files))
+        for (const QString &helpFile : versionHelpDir.entryList(QStringList("q*.qch"), QDir::Files))
             files.append({docPath, helpFile});
     }
     return files;
 }
 
-static QStringList documentationFiles(const QtVersions &vs, bool highestOnly = false)
+static AllDocumentationFiles allDocumentationFiles(const QtVersions &versions)
+{
+    AllDocumentationFiles result;
+    for (QtVersion *v : versions)
+        result.insert(v, allDocumentationFiles(v));
+    return result;
+}
+
+static QStringList documentationFiles(const QtVersions &vs,
+                                      const AllDocumentationFiles &allDocumentationFiles,
+                                      bool highestOnly = false)
 {
     // if highestOnly is true, register each file only once per major Qt version, even if
     // multiple minor or patch releases of that major version are installed
@@ -523,7 +546,8 @@ static QStringList documentationFiles(const QtVersions &vs, bool highestOnly = f
     for (QtVersion *v : versions) {
         const int majorVersion = v->qtVersion().majorVersion();
         QSet<QString> &majorVersionFileNames = includedFileNames[majorVersion];
-        for (const std::pair<Path, FileName> &file : documentationFiles(v)) {
+        const DocumentationFiles files = allDocumentationFiles.value(v);
+        for (const std::pair<Path, FileName> &file : files) {
             if (!highestOnly || !majorVersionFileNames.contains(file.second)) {
                 filePaths.insert(file.first + file.second);
                 majorVersionFileNames.insert(file.second);
@@ -533,15 +557,23 @@ static QStringList documentationFiles(const QtVersions &vs, bool highestOnly = f
     return filePaths.values();
 }
 
+static QStringList documentationFiles(const QtVersions &vs)
+{
+    return documentationFiles(vs, allDocumentationFiles(vs));
+}
+
 void QtVersionManagerImpl::updateDocumentation(const QtVersions &added,
                                                const QtVersions &removed,
-                                               const QtVersions &allNew)
+                                               const QtVersions &allNew,
+                                               bool updateBlockedDocumentation)
 {
     using DocumentationSetting = QtVersionManager::DocumentationSetting;
     const DocumentationSetting setting = QtVersionManager::documentationSetting();
+    const AllDocumentationFiles allNewDocFiles = allDocumentationFiles(allNew);
     const QStringList docsOfAll = setting == DocumentationSetting::None
                                       ? QStringList()
                                       : documentationFiles(allNew,
+                                                           allNewDocFiles,
                                                            setting
                                                                == DocumentationSetting::HighestOnly);
     const QStringList docsToRemove = Utils::filtered(documentationFiles(removed),
@@ -552,6 +584,17 @@ void QtVersionManagerImpl::updateDocumentation(const QtVersions &added,
                                                   [&docsOfAll](const QString &f) {
                                                       return docsOfAll.contains(f);
                                                   });
+
+    if (updateBlockedDocumentation) {
+        // The online installer registers documentation for Qt versions explicitly via an install
+        // setting, which defeats that we only register the Qt versions matching the setting.
+        // So the Qt support explicitly blocks the files that we do _not_ want to register, so the
+        // Help plugin knows about this.
+        const QSet<QString> reallyAllFiles = toSet(documentationFiles(allNew, allNewDocFiles));
+        const QSet<QString> toBlock = reallyAllFiles - toSet(docsOfAll);
+        Core::HelpManager::setBlockedDocumentation(toList(toBlock));
+    }
+
     Core::HelpManager::unregisterDocumentation(docsToRemove);
     Core::HelpManager::registerDocumentation(docsToAdd);
 }
